// AudioLowLevel: low level functions for AudioCapture and AudioPlay

using System;
using System.Text;
using System.IO;
using System.Collections;

using System.Runtime.InteropServices;


namespace AudioLowLevel
{

    // Encapsulates Wave file properties used internally be AudioIn and AudioOut.
    public class Wave
    {
        // Flag specifying the use of a callback FARPROC for sound messages
        public const int CALLBACK_FUNCTION = 0x00030000;
     
        // Structures and constants 
        public const int MM_WIM_DATA = 0x3C0;
        public const int MM_WOM_DONE = 0x3BD;

        [StructLayout(LayoutKind.Sequential)]
        public struct PCMWAVEFORMAT
        {
            public short wFormatTag;
            public short nChannels;
            public int nSamplesPerSec;
            public int nAvgBytesPerSec;
            public short nBlockAlign;
            public short wBitsPerSample;
            public short cbSize;

            public PCMWAVEFORMAT(short rate, short bits)
            {
                wFormatTag = 1;
                nChannels = 1;
                nSamplesPerSec = rate; //22050;
                wBitsPerSample = bits; //16;
                cbSize = 0;
                nBlockAlign = (short) (bits / 8); //(short) (nChannels * (bits / 8));
                nAvgBytesPerSec = (int) rate * nBlockAlign; //44100;
            }
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct WAVEHDR
        {
            public IntPtr lpData; // pointer to locked data buffer
            public int dwBufferLength; // length of data buffer
            public int dwBytesRecorded; // used for input only
            public IntPtr dwUser; // for client's use -> used to pass reference for BlockDone()
            public int dwFlags; // assorted flags (see defines)
            public int dwLoops; // loop control counter
            public IntPtr lpNext; // PWaveHdr, reserved for driver
            public int reserved; // reserved for driver
        
        }

        ///// waveIn section /////
        // Callback to catch unmanaged WM messages
        public delegate void DoneCallback(IntPtr hdrvr, int uMsg, int dwUser, ref WAVEHDR wavhdr, int dwParam2);
        [DllImport("winmm.dll")]
        public static extern int waveInOpen(ref IntPtr phwi, int uDeviceID, ref PCMWAVEFORMAT pwfx, DoneCallback dwCallback, int dwInstance, int fdwOpen);
       
        [DllImport("winmm.dll")]
        public static extern uint waveInClose(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveInReset(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveInStart(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveInStop(IntPtr hwi);

        [DllImport("winmm.dll")]
        public static extern int waveInGetNumDevs();
        [DllImport("winmm.dll")]
        public static extern int waveInGetDevCaps(uint uDeviceID, byte[] pwic, uint cbwic);

        //		typedef struct 
        //		{
        //			WORD wMid; 
        //			WORD wPid; 
        //			MMVERSION vDriverVersion; 
        //			TCHAR szPname[MAXPNAMELEN]; 
        //			DWORD dwFormats; 
        //			WORD wChannels; 
        //			WORD wReserved1;} 
        //		WAVEINCAPS;
        public class WAVEINCAPS
        {
            const uint WAVEINCAPS_SIZE = 80;

            private byte[] m_data = null;
            public uint Size { get { return (uint)WAVEINCAPS_SIZE; } }
            public ushort wMid { get { return BitConverter.ToUInt16(m_data, 0); } }
            public ushort wPid { get { return BitConverter.ToUInt16(m_data, 2); } }
            public uint vDriverVersion { get { return BitConverter.ToUInt32(m_data, 4); } }
            public uint dwFormats { get { return BitConverter.ToUInt32(m_data, 72); } }
            public ushort wChannels { get { return BitConverter.ToUInt16(m_data, 76); } }
            public ushort wReserved1 { get { return BitConverter.ToUInt16(m_data, 78); } }
            public string szPname
            {
                get
                {
                    char[] bytes = new char[32];
                    for (int i = 0; i < 32; i++)
                    {
                        bytes[i] = (char)BitConverter.ToUInt16(m_data, i * 2 + 8);
                    }

                    return new string(bytes);
                }
            }

            public WAVEINCAPS()
            {
                m_data = new byte[WAVEINCAPS_SIZE];
            }

            public static implicit operator byte[](WAVEINCAPS caps)
            {
                return caps.m_data;
            }
        }

        [DllImport("winmm.dll")]
        public static extern uint waveInAddBuffer(IntPtr hwi, ref WAVEHDR pwh, uint cbwh);
        [DllImport("winmm.dll")]
        public static extern uint waveInPrepareHeader(IntPtr hwi, ref WAVEHDR pwh, uint cbwh);
        [DllImport("winmm.dll")]
        public static extern uint waveInUnprepareHeader(IntPtr hwi, ref WAVEHDR pwh, uint cbwh);

        [DllImport("winmm.dll")]
        public static extern uint waveInGetVolume(IntPtr hwi, ref int vol);
        [DllImport("winmm.dll")]
        public static extern uint waveInSetVolume(IntPtr hwi, int vol);


        ///// waveOut section /////
        [DllImport("winmm.dll")]
        public static extern int waveOutOpen(ref IntPtr phwi, int uDeviceID, ref PCMWAVEFORMAT pwfx, DoneCallback dwCallback, int dwInstance, int fdwOpen);
        [DllImport("winmm.dll")]
        public static extern int waveOutWrite(IntPtr phwi, ref WAVEHDR pwh, uint cbwh);

        [DllImport("winmm.dll")]
        public static extern uint waveOutClose(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveOutReset(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveOutRestart(IntPtr hwi);
        [DllImport("winmm.dll")]
        public static extern uint waveOutPause(IntPtr hwi);

        [DllImport("winmm.dll")]
        public static extern uint waveOutPrepareHeader(IntPtr hwi, ref WAVEHDR pwh, uint cbwh);
        [DllImport("winmm.dll")]
        public static extern uint waveOutUnprepareHeader(IntPtr hwi, ref WAVEHDR pwh, uint cbwh);

        [DllImport("winmm.dll")]
        public static extern uint waveOutGetVolume(IntPtr hwi, ref int vol);
        [DllImport("winmm.dll")]
        public static extern uint waveOutSetVolume(IntPtr hwi, int vol);

    } // Wave ends


    // AudioDevice controls volume and devices
    public class Device
    {
        uint curNum = 0;

        // a struct to return the next device
        public struct aDevice
        {
            public int num;
            public string name;
        }

        public aDevice getNextDevice()
        {
            aDevice adev = new aDevice();
            adev.num = -1;

            uint num = (uint)Wave.waveInGetNumDevs();
            if (curNum == num)
                return adev;

            Wave.WAVEINCAPS caps = new Wave.WAVEINCAPS();
            Wave.waveInGetDevCaps(curNum, caps, caps.Size);

            adev.num = (int) curNum++;
            adev.name = caps.szPname;

            return adev;
        }
        /*
        public int getInVolume()
        {
            return 0;
        }

        public void setInVolume(int vol)
        {
        }
        */
    }
}
