// MouseCapture

using System;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace AudioIn
{
    // MouseCapture: gets mouse icon and position
    public class MouseCapture
    {
        private byte[] mouseRef;
        private Rectangle area;
        private short counter;
        private short maxcounter;

        public MouseCapture(Rectangle narea, int refperiod)
        {
            this.area = narea;
            this.mouseRef = new byte[4];
            this.mouseRef[0] = this.mouseRef[1] = this.mouseRef[2] = this.mouseRef[3] = 0;

            // refperiod is the fps period of image, used to set cursor update period
            this.counter = this.maxcounter = (short)(refperiod / 10);
        }

        public bool getPosition(ref Point pnt)
        {
            pnt = Cursor.Position;

            // if cursor out of capture window return false
            if (area.Contains(pnt))
            {
                pnt.X -= area.X;
                pnt.Y -= area.Y;
                return true;
            }
            else
                return false;
        }

        /* // Doesn`t works, capture only cursor over current form
		public bool getPointer(ref byte[] mouseCursor)
		{
            MemoryStream tms = new MemoryStream();

            this.mouseHandle = Cursor.Current.Handle;
            Icon.FromHandle(mouseHandle).Save(tms);
            mouseCursor = tms.ToArray();

            this.mouseIcon.ToBitmap().Save(tms, ImageFormat.Bmp);

            bool flag = false;
            // if cursor icon changed from last call return true
            if (this.mouseRef == mouseCursor)
            {
                return false;
            }
            else
            {

                this.mouseRef = mouseCursor;
                return true;
            }
		} */

        // so capture cursor icon with Win32 calls...

        public const Int32 CURSOR_SHOWING = 0x00000001;

        [StructLayout(LayoutKind.Sequential)]
        public struct ICONINFO
        {
            public bool fIcon;         // Specifies whether this structure defines an icon or a cursor. A value of TRUE specifies 
            public Int32 xHotspot;     // Specifies the x-coordinate of a cursor's hot spot. If this structure defines an icon, the hot 
            public Int32 yHotspot;     // Specifies the y-coordinate of the cursor's hot spot. If this structure defines an icon, the hot 
            public IntPtr hbmMask;     // (HBITMAP) Specifies the icon bitmask bitmap. If this structure defines a black and white icon, 
            public IntPtr hbmColor;    // (HBITMAP) Handle to the icon color bitmap. This member can be optional if this 
        }
        [StructLayout(LayoutKind.Sequential)]
        public struct POINT
        {
            public Int32 x;
            public Int32 y;
        }
        [StructLayout(LayoutKind.Sequential)]
        public struct CURSORINFO
        {
            public Int32 cbSize;        // Specifies the size, in bytes, of the structure. 
            public Int32 flags;         // Specifies the cursor state. This parameter can be one of the following values:
            public IntPtr hCursor;          // Handle to the cursor. 
            public POINT ptScreenPos;       // A POINT structure that receives the screen coordinates of the cursor. 
        }
        [DllImport("user32.dll", EntryPoint = "GetCursorInfo")]
        public static extern bool GetCursorInfo(out CURSORINFO pci);
        [DllImport("user32.dll", EntryPoint = "CopyIcon")]
        public static extern IntPtr CopyIcon(IntPtr hIcon);
        [DllImport("user32.dll", EntryPoint = "GetIconInfo")]
        public static extern bool GetIconInfo(IntPtr hIcon, out ICONINFO piconinfo);

        // return true if it has changed from last call
        public bool getPointer(ref byte[] mouseCursor)
        {
            mouseCursor = new byte[0];

            if (this.counter-- < 0)
            {
                MemoryStream tms = new MemoryStream();

                IntPtr hicon;
                CURSORINFO ci = new CURSORINFO();
                ICONINFO icInfo;
                ci.cbSize = Marshal.SizeOf(ci);

                if (GetCursorInfo(out ci))
                {
                    if (ci.flags == CURSOR_SHOWING)
                    {
                        hicon = CopyIcon(ci.hCursor);
                        if (GetIconInfo(hicon, out icInfo))
                        {
                            Icon.FromHandle(hicon).Save(tms);
                            mouseCursor = tms.ToArray();
                        }
                    }
                }

                // if cursor icon changed from last call return true, try to know changing upon 4 critical byte values
                if (mouseCursor.Length > 700)
                {
                    if (this.mouseRef[0] == mouseCursor[335] && this.mouseRef[1] == mouseCursor[340] && this.mouseRef[2] == mouseCursor[355] && this.mouseRef[3] == mouseCursor[356])
                        return false;
                    else
                    {
                        this.mouseRef[0] = mouseCursor[335];
                        this.mouseRef[1] = mouseCursor[340];
                        this.mouseRef[2] = mouseCursor[355];
                        this.mouseRef[3] = mouseCursor[356];

                        return true;
                    }
                }
                this.counter = this.maxcounter;
            }

            return true;
        }
    }
}