// Ermes Main.cs
//

using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

using System.Runtime.InteropServices;
using System.Threading;
using System.Text;
using System.Resources;

using Mux; // Put together audio video and mouse and send data

namespace Ermes
{
    // struct containing all settings
    public struct configStr
    {
        // server or client mode?
        private bool serverMode;

        // network stuff
        public string ipAddress;
        public int aPort;
        public int vPort;

        // section of screen to capture
        public Rectangle area;

        // audio / video
        public int devNum;
        public short audioSamples;
        public short audioBits;
        public uint blockAudioBuffer; // must be multiple of 2
        public int playoutBuffer;

        public bool kfqual;
        public ushort kfcnt;
        public int refperiod;
        public int fps;

        // constructor of struct
        public configStr(bool mode, Rectangle narea, string nipAddress, int aport, int vport)
        {
            this.serverMode = mode;
            this.area = narea;
            this.ipAddress = nipAddress;
            this.aPort = aport;
            this.vPort = vport;

            // static settings 
            this.devNum = -1; // the system default device
            this.audioSamples = 22050;
            this.audioBits = 16;
            this.blockAudioBuffer = 20480; // not less than 5120, not more than 51200 (see videoMuxer.syncQueue)
            this.playoutBuffer = 3000;

            // default settings
            this.kfqual = true; // true is high quality for keyframe
            this.kfcnt = 10; // a key frame each ten frames
            this.refperiod = 166; // (3fps = 333, 6fps = 166, 9fps = 111, 12fps = 83)
            this.fps = 6;
        }

        #region mode
        public void setServerMode()
        {
            this.serverMode = true;
        }
        public void setClientMode()
        {
            this.serverMode = false;
        }
        public bool isServerMode()
        {
            return this.serverMode;
        }
        #endregion

        #region video
        // sets video mode quality parameters
        public short SetVQuality(ushort kfcnt, bool kfqual, int refperiod, int fps)
        {
            this.kfcnt = kfcnt;
            this.kfqual = kfqual;
            this.refperiod = refperiod;
            this.fps = fps;

            return this.getBandW();
        }

        public short getBandW()
        {
            // calculate bandwith accordingly (0 450 ~ 3 1300)
            // sperimentally measured, reference image: static 1024x768

            if (kfqual)
            { // HQ
                if (kfcnt < 10)
                {
                    if (refperiod < 112)
                        return 3;
                    else
                        return 2;
                }
                else
                    if (kfcnt < 35)
                    {
                        if (refperiod < 112)
                            return 2;
                        else
                            return 1;
                    }
                    else
                    {
                        if (refperiod < 200)
                            return 2;
                        else
                            return 0;
                    }
            }
            else
            { // LQ
                if (kfcnt < 20)
                {
                    if (refperiod < 112)
                        return 2;
                    else
                        return 1;
                }
                else
                {
                    if (refperiod < 112)
                        return 1;
                    else
                        return 0;
                }
            }
        }

        public void setArea(Rectangle narea)
        {
            this.area = narea;
        }

        // easy to fill textboxes
        public string getX0()
        {
            return this.area.X.ToString();
        }
        public string getY0()
        {
            return this.area.Y.ToString();
        }
        public string getX1()
        {
            return (this.area.X + this.area.Width).ToString();
        }
        public string getY1()
        {
            return (this.area.Y + this.area.Height).ToString();
        }
        #endregion

        #region network
        public void setNetwork(string nipAddress, int aport, int vport)
        {
            this.ipAddress = nipAddress;
            this.aPort = aport;
            this.vPort = vport;
        }
        #endregion
    }

    // main menu
	public class Ermes : System.Windows.Forms.Form
	{

		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ContextMenu contextMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.NotifyIcon ErmesIcon;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem menuItem6;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem7;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.MenuItem menuItem12;
		private System.Windows.Forms.MenuItem menuItem14;
		private System.Windows.Forms.MenuItem menuItem15;
		private System.Windows.Forms.MenuItem menuItem13;
        private System.Windows.Forms.MenuItem menuItem16;

		private Thread CfgThr;
		private System.Threading.Timer timer0;
        private bool timerState = true;
        private Icon ico0;
        private Icon ico1;
        private Icon ico2;

		// struct containing all settings 
		private configStr currentConfig;

        // dialogs
        private About abox;
        private Help hbox;

        // Encoders and decoders
        private AudioMuxer aMuxer;
        private AudioDemuxer aDemuxer;
        private VideoMuxer vMuxer;
        private VideoDemuxer vDemuxer;

        ///// all starts here /////
        [STAThread]
        static void Main()
        {
            Application.Run(new Ermes());
        }

        public Ermes()
		{
			InitializeComponent();

            // starts loading form
            LoadForm lf = new LoadForm();
            lf.Show();
            lf.BringToFront();
            lf = null;

            // load icons from embedded resources
            ResourceManager resMan = new ResourceManager("Ermes.Ermes.Main.resx", GetType().Assembly);
            ico0 = new Icon(GetType(), "Graphics.Running0.ico");
            ico1 = new Icon(GetType(), "Graphics.Running1.ico");
            ico2 = new Icon(GetType(), "Graphics.Ermes.ico");

			// sets default configuration
			this.currentConfig = new configStr(
				false,	// starts in client mode
				Screen.PrimaryScreen.Bounds,	// capture fullscreen
				"224.1.1.1",	// multicast address
				1234, 1235);	// default ports

		}

		protected override void Dispose(bool disposing){
			if(disposing)
				if(components != null)
					components.Dispose();
			base.Dispose(disposing);
		}

		#region graphic stuff
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(Ermes));
            this.contextMenu1 = new System.Windows.Forms.ContextMenu();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.menuItem7 = new System.Windows.Forms.MenuItem();
            this.menuItem8 = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuItem14 = new System.Windows.Forms.MenuItem();
            this.menuItem13 = new System.Windows.Forms.MenuItem();
            this.menuItem16 = new System.Windows.Forms.MenuItem();
            this.menuItem15 = new System.Windows.Forms.MenuItem();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.menuItem9 = new System.Windows.Forms.MenuItem();
            this.menuItem10 = new System.Windows.Forms.MenuItem();
            this.menuItem11 = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.ErmesIcon = new System.Windows.Forms.NotifyIcon(this.components);
            this.SuspendLayout();
            // 
            // contextMenu1
            // 
            this.contextMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem5,
            this.menuItem6,
            this.menuItem14,
            this.menuItem15,
            this.menuItem1,
            this.menuItem2,
            this.menuItem3,
            this.menuItem4,
            this.menuItem11,
            this.menuItem12});
            // 
            // menuItem5
            // 
            this.menuItem5.Index = 0;
            this.menuItem5.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem7,
            this.menuItem8});
            this.menuItem5.Text = "Ermes v1.0 Client...";
            // 
            // menuItem7
            // 
            this.menuItem7.Index = 0;
            this.menuItem7.Text = "About...";
            this.menuItem7.Click += new System.EventHandler(this.aboutBox);
            // 
            // menuItem8
            // 
            this.menuItem8.Index = 1;
            this.menuItem8.Text = "Help...";
            this.menuItem8.Click += new System.EventHandler(this.helpBox);
            // 
            // menuItem6
            // 
            this.menuItem6.Index = 1;
            this.menuItem6.Text = "-";
            // 
            // menuItem14
            // 
            this.menuItem14.Index = 2;
            this.menuItem14.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem13,
            this.menuItem16});
            this.menuItem14.Text = "Run...";
            // 
            // menuItem13
            // 
            this.menuItem13.Index = 0;
            this.menuItem13.Text = "Start";
            this.menuItem13.Click += new System.EventHandler(this.RunStart);
            // 
            // menuItem16
            // 
            this.menuItem16.Enabled = false;
            this.menuItem16.Index = 1;
            this.menuItem16.Text = "Stop";
            this.menuItem16.Click += new System.EventHandler(this.RunStop);
            // 
            // menuItem15
            // 
            this.menuItem15.Index = 3;
            this.menuItem15.Text = "-";
            // 
            // menuItem1
            // 
            this.menuItem1.Enabled = false;
            this.menuItem1.Index = 4;
            this.menuItem1.Text = "Set client mode";
            this.menuItem1.Click += new System.EventHandler(this.ClientMode);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 5;
            this.menuItem2.Text = "Set server mode";
            this.menuItem2.Click += new System.EventHandler(this.ServerMode);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 6;
            this.menuItem3.Text = "-";
            // 
            // menuItem4
            // 
            this.menuItem4.Index = 7;
            this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem9,
            this.menuItem10});
            this.menuItem4.Text = "Settings...";
            // 
            // menuItem9
            // 
            this.menuItem9.Enabled = false;
            this.menuItem9.Index = 0;
            this.menuItem9.Text = "Configure server mode";
            this.menuItem9.Click += new System.EventHandler(this.StartConfig);
            // 
            // menuItem10
            // 
            this.menuItem10.Index = 1;
            this.menuItem10.Text = "Configure client mode";
            this.menuItem10.Click += new System.EventHandler(this.StartConfig);
            // 
            // menuItem11
            // 
            this.menuItem11.Index = 8;
            this.menuItem11.Text = "-";
            // 
            // menuItem12
            // 
            this.menuItem12.Index = 9;
            this.menuItem12.Text = "Quit";
            this.menuItem12.Click += new System.EventHandler(this.Quit);
            // 
            // ErmesIcon
            // 
            this.ErmesIcon.ContextMenu = this.contextMenu1;
            this.ErmesIcon.Icon = ((System.Drawing.Icon)(resources.GetObject("ErmesIcon.Icon")));
            this.ErmesIcon.Text = "Ermes v1.0\nClient mode";
            this.ErmesIcon.Visible = true;
            this.ErmesIcon.DoubleClick += new System.EventHandler(this.StartConfig);
            // 
            // Ermes
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.SystemColors.Control;
            this.ClientSize = new System.Drawing.Size(104, 171);
            this.ContextMenu = this.contextMenu1;
            this.ControlBox = false;
            this.Enabled = false;
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "Ermes";
            this.Opacity = 0;
            this.ShowInTaskbar = false;
            this.Text = "Server";
            this.WindowState = System.Windows.Forms.FormWindowState.Minimized;
            this.ResumeLayout(false);

		}
		#endregion

		#region methods to start secondary indipendent forms
		private void ExecServConfig()
		{
			ServConfig sc = new ServConfig(ref this.currentConfig);
			Application.Run(sc);
			this.currentConfig = sc.currentConfig;
			sc = null;
		}

		private void ExecClientConfig()
		{
			ClientConfig cc = new ClientConfig(ref this.currentConfig);
			Application.Run(cc);
			this.currentConfig = cc.currentConfig;
			cc = null;
		}
		#endregion

		private void BlinkIcon(object sender)
		{
            if (this.timerState)
            {
                this.ErmesIcon.Icon = ico0;
                this.timerState = false;
            }
            else
            {
                this.ErmesIcon.Icon = ico1;
                this.timerState = true;
            }
		}

		#region event handler functions
		private void StartConfig(object sender, System.EventArgs e)
		{
			if(this.currentConfig.isServerMode())
			{
				this.CfgThr = new Thread(new ThreadStart(this.ExecServConfig));
				this.CfgThr.Start();
			} 
			else 
			{
				this.CfgThr = new Thread(new ThreadStart(this.ExecClientConfig));
				this.CfgThr.Start();
			}

			this.CfgThr.Join();
		}

		private void aboutBox(object sender, System.EventArgs e)
		{
            this.abox = new About();
            this.abox.Show();
        }

		private void helpBox(object sender, System.EventArgs e)
		{
            this.hbox = new Help();
            this.hbox.Show();
		}
		
		private void ServerMode(object sender, System.EventArgs e)
		{
			if(this.CfgThr != null)
			{
				if(this.CfgThr.IsAlive)	
				{
					MessageBox.Show("Please close the configuration panel before switching mode.",
						"", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					return;
				}
				this.CfgThr.Join();
			}

            // stops playing before switching mode
            this.ClientStop();

			this.currentConfig.setServerMode();
			// update of menu
			this.menuItem1.Enabled = true;
			this.menuItem2.Enabled = false;
			this.menuItem9.Enabled = true;
			this.menuItem10.Enabled = false;
			this.ErmesIcon.Text = "Ermes v1.0\nServer mode";
			this.menuItem5.Text = "Ermes v1.0 Server...";
            this.ErmesIcon.Icon = ico1;
	
		}

		private void ClientMode(object sender, System.EventArgs e)
		{
			if(this.CfgThr != null)
			{
				if(this.CfgThr.IsAlive)	
				{
					MessageBox.Show("Please close the configuration panel before switching mode.",
						"", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Exclamation);
					return;
				}
				this.CfgThr.Join();
			}

			// stops capture before switching
			this.ServerStop();

			this.currentConfig.setClientMode();
			// update of menu
			this.menuItem2.Enabled = true;
			this.menuItem1.Enabled = false;
			this.menuItem9.Enabled = false;
			this.menuItem10.Enabled = true;
			this.ErmesIcon.Text = "Ermes v1.0\nClient mode";
			this.menuItem5.Text = "Ermes v1.0 Client...";
            this.ErmesIcon.Icon = ico2;
		}


		// in server mode starts capturing and sending through the network
		private void RunStart(object sender, System.EventArgs e)
		{
            if (this.currentConfig.isServerMode())
            {   // I am in capture mode (server)

                // update menu and icon
                this.menuItem13.Enabled = false;
                this.menuItem16.Enabled = true;
                this.ErmesIcon.Text = "Ermes v1.0\nServer mode\nNow capturing...";

                // blinking icon stuff
                TimerCallback tcb0 = new TimerCallback(this.BlinkIcon);
                this.timer0 = new System.Threading.Timer(tcb0, null, 0, 667);

                try
                {
                    this.aMuxer = new AudioMuxer(this.currentConfig.area, this.currentConfig.ipAddress,
                        this.currentConfig.aPort, this.currentConfig.devNum, this.currentConfig.audioSamples,
                        this.currentConfig.audioBits, this.currentConfig.blockAudioBuffer, this.currentConfig.refperiod);

                    this.vMuxer = new VideoMuxer(this.currentConfig.area, this.currentConfig.kfqual,
                        this.currentConfig.kfcnt, this.currentConfig.refperiod, this.currentConfig.ipAddress,
                        this.currentConfig.vPort);

                }
                catch (Exception excep)
                {
                    MessageBox.Show("An error occurred while encoding or sending data.\n" + excep.Message, "",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.RunStop(this, null);
                }
                
            }
            else
            {   // I am in client mode

                // update menu 
                this.menuItem13.Enabled = false;
                this.menuItem16.Enabled = true;

                this.ErmesIcon.Text = "Ermes v1.0\nClient mode\nNow playing...";

                // set up the queue for sincronizing video with audio
                System.Collections.Queue tsqueue = new System.Collections.Queue(4, 1); //.Synchronized(new System.Collections.Queue(4,1));
                
                try
                {
                    this.aDemuxer = new AudioDemuxer(tsqueue, this.currentConfig.ipAddress,
                        this.currentConfig.aPort, this.currentConfig.devNum, this.currentConfig.audioSamples,
                        this.currentConfig.audioBits, this.currentConfig.playoutBuffer);

                    this.vDemuxer = new VideoDemuxer(tsqueue, this.currentConfig.ipAddress, this.currentConfig.vPort);

                }
                catch (Exception excep)
                {
                    MessageBox.Show("Problem in receiving or decoding data.\n" + excep.ToString(), "",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.RunStop(this, null);
                }

            }
        }

		// in server mode stops capture and network
		private void RunStop(object sender, System.EventArgs e)
		{
            if (this.currentConfig.isServerMode())
            {
                this.ServerStop();
            }
            else
            {
                this.ClientStop();
            }
		}

        // painless method to stops server capture activity
        private void ServerStop()
        {
            // if the capture is really active
            if (this.aMuxer != null)
            {
                // stops audio muxer
                this.aMuxer.AudioInStop();
                this.aMuxer = null;

                // disable blinking icon
                this.timer0.Dispose();
                this.timer0 = null;
            }

            if (this.vMuxer != null)
            {
                this.vMuxer.Stop();
                this.vMuxer = null;
            }

            this.ErmesIcon.Icon = ico1;
            this.ErmesIcon.Text = "Ermes v1.0\nServer mode";
            this.menuItem13.Enabled = true;
            this.menuItem16.Enabled = false;

        }

        // painless method to stop client window
        private void ClientStop()
        {
            // if the client is really active
            if (this.aDemuxer != null)
            {
                // stops audio player
                this.aDemuxer.Stop();
                this.aDemuxer = null;
            }
            
            if (this.vDemuxer != null)
            {
                // stops audio player
                this.vDemuxer.Stop();
                this.vDemuxer = null;
            }
            
            this.ErmesIcon.Icon = ico2;
            this.ErmesIcon.Text = "Ermes v1.0\nClient mode";
            this.menuItem13.Enabled = true;
            this.menuItem16.Enabled = false;
        }

		// cleaning before exiting
		private void Quit(object sender, System.EventArgs e)
		{
			// stops activities
			if(this.currentConfig.isServerMode())
                this.ServerStop();
			else
				this.ClientStop();

			// there`s a config window opened?
			if(this.CfgThr != null)
			{
				if(this.CfgThr.IsAlive)	
				{
					MessageBox.Show("Please close the settings panel before quitting.",
						"", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Exclamation);
					return;
				}
				this.CfgThr.Join();
			}

            // close dialogs
            this.abox = null;
            this.hbox = null;

			this.Dispose();
		}

		#endregion

	}


}
