
using System;
using System.Text;
using System.IO;
using System.Threading;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;

using Network;
using AudioIn;

namespace Mux
{

    // AudioMuxer.cs manages audio packet structures and buffers
    public class AudioMuxer
    {
        private AudioCapture ain;
        private MouseCapture mCapt;
        private NetServer netServ;

        private Thread audioInThr;
        private ushort counter;
        private int currentDevice;
        private short samples;
        private short bits;
        private uint audioBuf;

        // The delegate to call my member AudioMuxer.BuildPacket() when AudioIn change buffer
        AudioCapture.BufferReadyDelegate BufferReady;


        public AudioMuxer(Rectangle captureArea, string addr, int port, int nAudioDev, short nSamples, short nBits, uint nAudioBuf, int refperiod)
        {
            // Initialization
            this.counter = 0;
            this.audioInThr = null;
            this.currentDevice = nAudioDev;
            this.samples = nSamples;
            this.bits = nBits;
            this.audioBuf = nAudioBuf;

            this.mCapt = new MouseCapture(captureArea, refperiod);
            this.netServ = new NetServer(addr, port);

            BufferReady = new AudioCapture.BufferReadyDelegate(this.BuildPacket);

            // The thread managing AudioCapture class
            this.audioInThr = new Thread(new ThreadStart(this.AudioInStart));
            this.audioInThr.Start();
        }

        // The method that starts recording
        private void AudioInStart()
        {
            this.ain = new AudioCapture(BufferReady);
            if (!this.ain.Load(this.currentDevice, this.samples, this.bits, this.audioBuf))
                throw new Exception("A problem occured loading audio system.");

            this.ain.Start();
        }

        // Stops all audio-related process
        public void AudioInStop()
        {
            this.BufferReady = null;

            this.ain.Stop();
            if (this.ain != null)
                this.ain.Dispose();

            // I have to be sure the AudioIn thread is really terminated
            if (this.audioInThr != null)
                this.audioInThr.Join();
        }


        // Must be called from AudioIn as new event
        public void BuildPacket(byte[] buffer, int size)
        {
            ushort mouseLen = 0;
            byte[] mCursor = new byte[0];
            Point mousePnt = Point.Empty;
            bool toDraw = false;

            //Get the cursor position information, if the pointer is out of area don`t set anything
            if (toDraw = mCapt.getPosition(ref mousePnt))
            {
                // if the cursor has changed from last time get a byte[] representation
                if (mCapt.getPointer(ref mCursor))
                {
                    mouseLen = (ushort)mCursor.Length;
                }
            }
            
            // Build the struct
            AUDIOPACKET aPacket = new AUDIOPACKET(DateTime.UtcNow, size, buffer, mousePnt, mouseLen, mCursor, toDraw);

            byte[] pktBytes = aPacket.GetBytes();
            netServ.sendByte(pktBytes, pktBytes.Length, counter);

            if (counter == ushort.MaxValue)
                counter = 0;
            else
                counter++;
        }

    }

}
