// VideoMuxer

using System;
using System.Text;
using System.IO;
using System.Threading;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;

using Network;
using Video;

namespace Mux
{

    // VideoMuxer get frames and send them
    public class VideoMuxer
    {
        private VideoCapture vCapt;
        private NetServer netServ;

        private bool keyFrameLowQ;
        private ushort keyFrameCount;
        private int refreshTime;

        private bool isRunning;
        private ushort counter;
        private Rectangle area;
        private Thread captThr;

        // controls video encoding and make packets
        // keyFrameLowQ specifies if keyframes are high or low quality
        // keyframeCount controls number of frame between keyframes (1 = only keyframes)
        // refreshTime controls the fps (2fps = 500, 3fps = 333, 6fps = 166, 12fps = 83)
        public VideoMuxer(Rectangle area, bool keyFrameHighQ, ushort keyFrameCount, int refreshTime, string netAddr, int netPort)
        {
            this.area = area;
            this.keyFrameLowQ = !keyFrameHighQ;
            this.keyFrameCount = keyFrameCount;
            this.refreshTime = refreshTime;
            this.counter = 0;

            this.netServ = new NetServer(netAddr, netPort);
        
            captThr = new Thread(new ThreadStart(this.CaptureAndSend));
            captThr.Start();
        }

        public void Stop()
        {
            this.isRunning = false;
            this.captThr.Join();
            this.vCapt = null;
        }

        public void CaptureAndSend()
        {
            this.vCapt = new VideoCapture(this.area);
         
            this.isRunning = true;
            int frcount = 0;
            VIDEOPACKET vPacket;
            byte[] data;

            while (this.isRunning)
            {
                if (frcount == 0)
                {
                    data = this.vCapt.getKeyFrame(keyFrameLowQ);
                    frcount = this.keyFrameCount;

                    // Build the struct
                    vPacket = new VIDEOPACKET(DateTime.UtcNow, data.Length, data, true);
                }
                else
                {
                    data = this.vCapt.getInterFrame();

                    // Build the struct
                    vPacket = new VIDEOPACKET(DateTime.UtcNow, data.Length, data, false);      
                }


                // send the video packet
                byte[] pktBytes = vPacket.GetBytes();
                netServ.sendByte(pktBytes, pktBytes.Length, counter);

                frcount--;

                if (counter == ushort.MaxValue)
                    counter = 0;
                else
                    counter++;
                
                Thread.Sleep(this.refreshTime - 50); // 50 milliseconds of working time
            }
        }


    }
}
