using System;
using System.Net;
using System.Net.Sockets;

using System.IO;
using System.Collections;

namespace Network
{
    // a multicast udp receiver
    public class NetClient
    {
        private IPAddress groupAddress;
        private IPEndPoint endPointUDP;
        private UdpClient clientUDP;
        private bool isRunning;

        public NetClient(string addr, int port)
        {
            this.groupAddress = IPAddress.Parse(addr);
            this.endPointUDP = new IPEndPoint(groupAddress, port);
            this.clientUDP = new UdpClient(port);

            // the receiver needs to join the group, max ttl=2
            this.clientUDP.JoinMulticastGroup(this.groupAddress, 2);

            this.isRunning = true;
        }

        public void Close()
        {
            this.isRunning = false;

            if (this.clientUDP != null)
            {
                this.clientUDP.DropMulticastGroup(this.groupAddress);

                this.clientUDP.Close();
                this.endPointUDP = null;
                this.clientUDP = null;
            }
        }

        public void recvByte(ref byte[] bytes, ref int len)
        {
            Hashtable packetList = new Hashtable();
            MemoryStream ms;

            if (this.clientUDP != null)
            {
                while (this.isRunning)
                {
                    try
                    {
                        ms = new MemoryStream(this.clientUDP.Receive(ref this.endPointUDP));
                    }
                    catch (SocketException)
                    {
                        bytes = new byte[0];
                        len = 0;
                        return;
                    }

                    byte[] buffer = new byte[ms.Length - 4];
                    ms.Read(buffer, 0, 2);
                    ushort counter = (ushort)BitConverter.ToUInt16(buffer, 0);
                    byte ftot = (byte)ms.ReadByte();
                    byte fnum = (byte)ms.ReadByte();
                    ms.Read(buffer, 0, (int)ms.Length - 4);

                    if (packetList.ContainsKey(counter))
                        ((partialData)packetList[counter]).addData(ftot, fnum, buffer);
                    else
                        packetList.Add(counter, new partialData(ftot, fnum, buffer));

                    if (((partialData)packetList[counter]).isDone())
                    {
                        bytes = ((partialData)packetList[counter]).getData();
                        len = bytes.Length;

                        packetList.Remove(counter);
                        return;
                    }
                }
            }
            else
            {
                throw new Exception("Socket not initialized");
            }
        }


        // a structure to keep partially received data
        class partialData
        {
            private Hashtable fl;
            private byte totfrag;

            public partialData(byte totfrag, byte fragnum, byte[] data)
            {
                this.totfrag = totfrag;
                if (fl == null)
                    fl = new Hashtable(totfrag);

                fl.Add(fragnum, data);
            }

            public void addData(byte totfrag, byte fragnum, byte[] data)
            {
                this.totfrag = totfrag;
                if (fl == null)
                    fl = new Hashtable(totfrag);

                fl.Add(fragnum, data);
            }

            public bool isDone()
            {
                // is the packet complete?
                if (fl.Count == totfrag)
                    return true;
                return false;
            }

            public byte[] getData()
            {
                MemoryStream ms = new MemoryStream();

                for (byte i = 0; i < totfrag; i++)
                {
                    byte[] data = (byte[])fl[i];
                    ms.Write(data, 0, data.Length);
                }

                return ms.ToArray();
            }
        }
    }
}
