using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Design;
using System.Drawing.Imaging;

using System.IO;

using System.Runtime.InteropServices;

namespace Video
{
    // Low level funtions from GDI+ 
    internal class GDICapture
    {
        // gdi32.dll
        public const int SRCCOPY = 13369376;
        public const int CAPTUREBLT = 1073741824;

        [DllImport("gdi32.dll", EntryPoint = "DeleteDC")]
        public static extern IntPtr DeleteDC(IntPtr hDc);

        [DllImport("gdi32.dll", EntryPoint = "DeleteObject")]
        public static extern IntPtr DeleteObject(IntPtr hDc);

        [DllImport("gdi32.dll", EntryPoint = "BitBlt")]
        public static extern bool BitBlt(IntPtr hdcDest, int xDest, int yDest, int wDest, int hDest, IntPtr hdcSource,
            int xSrc, int ySrc, int RasterOp);

        [DllImport("gdi32.dll", EntryPoint = "CreateCompatibleBitmap")]
        public static extern IntPtr CreateCompatibleBitmap(IntPtr hdc, int nWidth, int nHeight);

        [DllImport("gdi32.dll", EntryPoint = "CreateCompatibleDC")]
        public static extern IntPtr CreateCompatibleDC(IntPtr hdc);

        [DllImport("gdi32.dll", EntryPoint = "SelectObject")]
        public static extern IntPtr SelectObject(IntPtr hdc, IntPtr bmp);

        // user32.dll
        [DllImport("user32.dll", EntryPoint="GetDesktopWindow")]
        public static extern IntPtr GetDesktopWindow();

        [DllImport("user32.dll",EntryPoint="GetDC")]
        public static extern IntPtr GetDC(IntPtr ptr);

        [DllImport("user32.dll",EntryPoint="GetWindowDC")]
        public static extern IntPtr GetWindowDC(Int32 ptr);

        [DllImport("user32.dll",EntryPoint="ReleaseDC")]
        public static extern IntPtr ReleaseDC(IntPtr hWnd,IntPtr hDc);


    }


	// image capturing library
	public class VideoCapture
	{
		private Rectangle area;
        
        private Rectangle workRect;
        private Bitmap lastBmp;
        
		public VideoCapture(Rectangle narea)
		{
			this.area = narea;

            // this is the area on the bitmap where we work. The size must be a multiple of 16
            workRect = new Rectangle(0, 0, (this.area.Width / 16) * 16, (this.area.Height / 16) * 16);
		}

        // grab the screen, using this.area
        private Bitmap ScreenCapt()
        {
            IntPtr hBitmap;
            // Get the device context handle
            IntPtr hDC = GDICapture.GetDC(GDICapture.GetDesktopWindow());
            IntPtr hMDC = GDICapture.CreateCompatibleDC(hDC);

            // The creation of the bitmap
            if ((hBitmap = GDICapture.CreateCompatibleBitmap(hDC, this.area.Width, this.area.Height)) == IntPtr.Zero)
                throw new Exception("Memory full");

            // hOld keep a bitmap
            IntPtr hOld = (IntPtr)GDICapture.SelectObject(hMDC, hBitmap);

            // Copy the portion of device screen to the bitmap handle
            GDICapture.BitBlt(hMDC, 0, 0, this.area.Width, this.area.Height, hDC, this.area.Left, this.area.Top,
                GDICapture.SRCCOPY | GDICapture.CAPTUREBLT);

            // Get the bitmap in Bitmap format
            Bitmap bmp = System.Drawing.Image.FromHbitmap(hBitmap);

            // hMDC back to hOld
            GDICapture.SelectObject(hMDC, hOld);

            // Clean resourses
            GDICapture.DeleteDC(hMDC);
            GDICapture.ReleaseDC(GDICapture.GetDesktopWindow(), hDC);
            GDICapture.DeleteObject(hBitmap);

            // This statement runs the garbage collector manually
            GC.Collect();

            return bmp;
        }
    
        // return the current bitmap as keyframe
		public byte[] getKeyFrame(bool lowQ)
		{
            MemoryStream ms = new MemoryStream();
            ImageCodecInfo[] ici = ImageCodecInfo.GetImageEncoders();
            EncoderParameters encparams = new EncoderParameters(1);
            if (lowQ)
                encparams.Param[0] = new EncoderParameter(Encoder.Quality, (long)35);
            else
                encparams.Param[0] = new EncoderParameter(Encoder.Quality, (long)90);

            this.lastBmp = this.ScreenCapt();
            // save as jpeg with quality 35 or 90
            this.lastBmp.Save(ms, ici[1], encparams);

            return (byte[])ms.ToArray();
		}

        // return the current bitmap as difference from the last frame
        public byte[] getInterFrame()
        {
            byte[] diff;
            Bitmap curBitmap;

            lock (this)
            {
                curBitmap = this.ScreenCapt();

                diff = VideoCompression.GetDiff(curBitmap, this.workRect, this.lastBmp);
                this.lastBmp = curBitmap;
            }
            return diff;
        }

	}
}



