using System;
using System.Drawing;
using System.Drawing.Design;
using System.Drawing.Imaging;

using System.Collections;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace Video
{
    // stores and reproduce images
    public partial class VideoPlayer : Form
    {
        private Rectangle area;
        private Rectangle rect;
        public bool isRunning;
        private bool isReady;
        private Bitmap lastFrame;
        public FrameList frames;
        private long tstamp;
        private Icon mCursorRef;

        // delegate used with Invoke()
        public delegate long ShowFrameDelegate(bool nDraw, int nCursorSize, byte[] nCursor, ushort nx, ushort ny);
        public delegate void AddFrameDelegate(long timestamp, byte[] framedata, bool isKeyframe);
        public ShowFrameDelegate sfDeleg;
        public AddFrameDelegate afDeleg;

        private PictureBox pictureBox1;
        private System.ComponentModel.IContainer components = null;

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.pictureBox1 = new System.Windows.Forms.PictureBox();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).BeginInit();
            this.SuspendLayout();
            // 
            // pictureBox1
            // 
            this.pictureBox1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.pictureBox1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pictureBox1.Location = new System.Drawing.Point(0, 0);
            this.pictureBox1.Name = "pictureBox1";
            this.pictureBox1.Size = new System.Drawing.Size(0, 0);
            this.pictureBox1.SizeMode = System.Windows.Forms.PictureBoxSizeMode.AutoSize;
            this.pictureBox1.TabIndex = 0;
            this.pictureBox1.TabStop = false;
            // 
            // VideoPlayer
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.AutoSize = true;
            this.AutoSizeMode = System.Windows.Forms.AutoSizeMode.GrowAndShrink;
            this.AutoValidate = System.Windows.Forms.AutoValidate.EnablePreventFocusChange;
            this.CausesValidation = false;
            this.ClientSize = new System.Drawing.Size(0, 0);
            this.ControlBox = false;
            this.Controls.Add(this.pictureBox1);
            this.DoubleBuffered = true;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.Name = "VideoPlayer";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Ermes Client Window";
            this.TopMost = true;
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).EndInit();
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        #endregion

        protected override void Dispose(bool disposing)
        {
            this.isRunning = false;
            this.isReady = false;

            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        public VideoPlayer()
        {
            this.InitializeComponent();
            this.mCursorRef = null;

            this.sfDeleg = new ShowFrameDelegate(this.ShowFrame);
            this.afDeleg = new AddFrameDelegate(this.AddFrame);

            this.frames = new FrameList();
            this.isReady = false;
            this.isRunning = true;
        }

        // set up regions and dialog after the first keyframe received
        private void setArea(int width, int height)
        {
            this.area = new Rectangle(0, 0, width, height);
            // this is the area on the bitmap where compression works. The size must be a multiple of 16
            this.rect = new Rectangle(0, 0, (width / 16) * 16, (height / 16) * 16);

           // this.pictureBox1.Width = width;
           // this.pictureBox1.Height = height;
            this.isReady = true;
        }

        // VideoDemuxer add here frames
        public void AddFrame(long timestamp, byte[] framedata, bool isKeyframe)
        {
            MemoryStream ms;

            if (isKeyframe)
            {
                ms = new MemoryStream(framedata);
                lastFrame = new Bitmap(ms);
            }
            else
            {
                // If I haven`t received yet a keyframe, don`t do anything
                if (this.isReady)
                    VideoCompression.GetBmp(framedata, this.rect, ref lastFrame);
                else
                    return;
            }

            // add bmp to list
            this.frames.Add(timestamp, lastFrame);
            // set the size for interframe calculations
            this.setArea(lastFrame.Width, lastFrame.Height);
        }

        // show the next frame contained in the hashtable. Returns the timestamp of showed frame
        public long ShowFrame(bool nDraw, int nCursorSize, byte[] nCursor, ushort nx, ushort ny)
        {
            if (this.isRunning && !this.frames.isEmpty())
            {
                Bitmap bmpfr = this.frames.GetNext(ref tstamp);

                // If the cursor is in area
                if (nDraw)
                {
                    Graphics grafr = Graphics.FromImage(bmpfr);

                    // build the cursor icon
                    if (nCursorSize > 0)
                    {
                        MemoryStream ms = new MemoryStream(nCursor);
                        Icon mCursor = new Icon(ms);
                        this.mCursorRef = mCursor;

                        grafr.DrawIcon(mCursor, nx - 10, ny - 10); // 10 px of icon border to remove
                    }
                    else
                    {
                        // I `ve lost the cursor information
                        if (this.mCursorRef != null)
                        {
                            grafr.DrawIcon(this.mCursorRef, nx - 10, ny - 10);
                        }
                        else
                        {   // last chance, I draw a dot
                            grafr.DrawEllipse(Pens.Black, nx - 2, ny - 2, 4, 4);
                        }
                    }
                }

                pictureBox1.Image = bmpfr;
                pictureBox1.Invalidate(); // force the redrawing of picturebox

                return tstamp;
            }
            else
            {
                // make the VideoDemuxer thread wait
                return -1;
            }
        }

    } // end class videoPlayer


    #region Manage the list of frames
    public class FrameList
    {
        private Hashtable ht;

        public FrameList()
        {
            ht = Hashtable.Synchronized(new Hashtable());
        }

        public void Add(long ts, Bitmap frame)
        {
            lock (ht)
            {
                try
                {
                    ht.Add(ts, frame);
                }
                catch
                {
                }
            }
        }

        // Returns the next frame, delete older if some arrived meanwhile
        public Bitmap GetNext(ref long ts)
        {
            Bitmap bmp;
            long firstKey = long.MaxValue;
            
            lock (ht)
            {
                ICollection keyColl = ht.Keys;

                foreach (long key in keyColl)
                    if (firstKey > key)
                        firstKey = key;

                bmp = (Bitmap)ht[firstKey];
                ht.Remove(firstKey);

                ts = firstKey;

                // clean the hashTable
                foreach (long key in keyColl)
                    if (key < firstKey)
                        ht.Remove(key);
            }

            return bmp;
        }

        internal bool isEmpty()
        {
            return (this.ht.Count == 0) ? true : false;
        }
    }
    #endregion

}
